#include "stdafx.h"
#include "ButtonTexture.h"

NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
		};

ButtonTexture::ButtonTexture(const string &name)
{
	memoryNormalDC = 0;
	memoryNormalBitmap = 0;

	memoryActiveDC = 0;
	memoryActiveBitmap = 0;

	memoryMinimizedDC = 0;
	memoryMinimizedBitmap = 0;

	memoryFlashingDC = 0;
	memoryFlashingBitmap = 0; 

	memoryHoverDC = 0;
	memoryHoverBitmap = 0; 

	/*has_normalAlphaMap = false;
	has_activeAlphaMap = false;
	has_minimizedAlphaMap = false;
	has_flashingAlphaMap = false;*/

	defaultbutton_saturation = GetxRCInt(name , "ButtonSaturationIntensity", 255, 0, 255);
	defaultbutton_hueIntensity = GetxRCInt(name , "ButtonHueIntensity", 0, 0, 255);
	defaultbutton_hueColor = GetxRCColor(name, "ButtonHueColor", RGB(255, 255, 255));

	defaultAlpha = GetxRCInt(name , "ButtonAlphaBlend", 255, 0, 255);

	defaultleftEdge = GetxRCInt(name, "ButtonImageLeftEdge", 0, 0);
	defaulttopEdge = GetxRCInt(name, "ButtonImageTopEdge", 0, 0);
	defaultrightEdge = GetxRCInt(name, "ButtonImageRightEdge", 0, 0);
	defaultbottomEdge = GetxRCInt(name, "ButtonImageBottomEdge", 0, 0);

	defaultmode = GetxRCNamedValue(name, "ButtonImageMode", modeConstants, MULTIBLT_STRETCH);

	defaultsolidcolors = GetxRCLine(name, "ButtonSolidColors", "ffffff");
	defaultbevelSize = GetxRCInt(name, "ButtonSolidBevelSize", 0, 0);
	defaultgradientColor = GetxRCLine(name, "ButtonSolidGradientColors", "");
	defaultverticalGradient = GetxRCBool(name, "ButtonSolidGradientVertical");
}

ButtonTexture::~ButtonTexture()
{
	NormalGradient.clear();
	ActiveGradient.clear();
	MinimizedGradient.clear();
	FlashingGradient.clear();

	HoverGradient.clear();

	if(memoryNormalBitmap)
	{
		memoryNormalBitmap = (HBITMAP) SelectObject(memoryNormalDC, memoryNormalBitmap);
		DeleteObject(memoryNormalBitmap);
	}
	if(memoryNormalDC)
		DeleteDC(memoryNormalDC);

	if(memoryActiveBitmap)
	{
		memoryActiveBitmap = (HBITMAP) SelectObject(memoryActiveDC, memoryActiveBitmap);
		DeleteObject(memoryActiveBitmap);
	}
	if(memoryActiveDC)
		DeleteDC(memoryActiveDC);

	if(memoryMinimizedBitmap)
	{
		memoryMinimizedBitmap = (HBITMAP) SelectObject(memoryMinimizedDC, memoryMinimizedBitmap);
		DeleteObject(memoryMinimizedBitmap);
	}
	if(memoryMinimizedDC)
		DeleteDC(memoryMinimizedDC);

	if(memoryFlashingBitmap)
	{
		memoryFlashingBitmap = (HBITMAP) SelectObject(memoryFlashingDC, memoryFlashingBitmap);
		DeleteObject(memoryFlashingBitmap);
	}
	if(memoryFlashingDC)
		DeleteDC(memoryFlashingDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);
}

void ButtonTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void ButtonTexture::parseGradientCFG(string cfg, int imageType)
{
	int color;
	COLORREF insertColor;
	char token[4096];
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			NormalGradient.insert(NormalGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			ActiveGradient.insert(ActiveGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			MinimizedGradient.insert(MinimizedGradient.end(), insertColor );
		}
		else if (imageType == 3)
		{
			FlashingGradient.insert(FlashingGradient.end(), insertColor );
		}
		else if (imageType == 4)
		{
			HoverGradient.insert(HoverGradient.end(), insertColor );
		}
	}
}


void ButtonTexture::configure(const string &name)
{
	string imageFile = "";

	usenormalSolidColors = GetxRCBool(name, "UseNormalButtonSolidColors", GetxRCBool(name, "UseButtonSolidColors"));

	if (usenormalSolidColors)
	{
		imageFile = GetxRCLine(name, "NormalButtonSolidColors", defaultsolidcolors);
		parseCFG(imageFile, normalbackgroundColor, normallightBevelColor, normaldarkBevelColor);
		//BG Insert in Gradient List
		NormalGradient.insert(NormalGradient.end(), normalbackgroundColor );

		normalbevelSize = GetxRCInt(name, "NormalButtonSolidBevelSize", defaultbevelSize, 0);

		normalgradientColor = GetxRCLine(name, "NormalButtonSolidGradientColors", defaultgradientColor);
		parseGradientCFG(normalgradientColor, 0);
		
		normalverticalGradient = GetxRCBool(name, "NormalButtonSolidGradientVertical", defaultverticalGradient);
	}
	else
	{
		imageFile = GetxRCString(name, "ButtonImage", "");
		if (imageFile.empty())
			imageFile = GetxRCString(name, "NormalButtonImage", "");
		else if (!imageFile.empty())
		{
			string temp = GetxRCString(name, "NormalButtonImage", "");
			if ( !temp.empty() )
				imageFile = temp;
		}

		memoryNormalBitmap = LoadLSImage(imageFile.c_str(), 0);

		/*if (memoryNormalBitmap)
		{
			BITMAP bm;
			GetObject(memoryNormalBitmap, sizeof(BITMAP), &bm);
			has_normalAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

			has_activeAlphaMap = has_minimizedAlphaMap = has_flashingAlphaMap = has_normalAlphaMap;

			if (has_normalAlphaMap)
				PrepareAlphaChannel(memoryNormalBitmap);
		}*/

		GetLSBitmapSize(memoryNormalBitmap, &imageNormalWidth, &imageNormalHeight);

		memoryNormalDC = CreateCompatibleDC(0);
		SelectObject(memoryNormalDC, memoryNormalBitmap);
	}

	useactiveSolidColors = GetxRCBool(name, "UseActiveButtonSolidColors", usenormalSolidColors);

	if (useactiveSolidColors)
	{
		imageFile = GetxRCLine(name, "ActiveButtonSolidColors", "ffffff");
		parseCFG(imageFile, activebackgroundColor, activelightBevelColor, activedarkBevelColor);
		//BG Insert in Gradient List
		ActiveGradient.insert(ActiveGradient.end(), activebackgroundColor );

		activebevelSize = GetxRCInt(name, "ActiveButtonSolidBevelSize", normalbevelSize, 0);

		parseGradientCFG(GetxRCLine(name, "ActiveButtonSolidGradientColors", normalgradientColor), 1);
		
		activeverticalGradient = GetxRCBool(name, "ActiveButtonSolidGradientVertical", normalverticalGradient);
	}
	else
	{
		imageFile = GetxRCString(name, "ActiveButtonImage", "");
		if (!imageFile.empty())
		{
			memoryActiveBitmap = LoadLSImage(imageFile.c_str(), 0);

			/*if (memoryActiveBitmap)
			{
				BITMAP bm;
				GetObject(memoryActiveBitmap, sizeof(BITMAP), &bm);
				has_activeAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_activeAlphaMap)
					PrepareAlphaChannel(memoryActiveBitmap);
			}*/

			GetLSBitmapSize(memoryActiveBitmap, &imageActiveWidth, &imageActiveHeight);

			memoryActiveDC = CreateCompatibleDC(0);
			SelectObject(memoryActiveDC, memoryActiveBitmap);
		}
	}

	useminimizedSolidColors = GetxRCBool(name, "UseMinimizedButtonSolidColors", usenormalSolidColors);

	if (useminimizedSolidColors)
	{
		imageFile = GetxRCLine(name, "MinimizedButtonSolidColors", "ffffff");
		parseCFG(imageFile, minimizedbackgroundColor, minimizedlightBevelColor, minimizeddarkBevelColor);
		//BG Insert in Gradient List
		MinimizedGradient.insert(MinimizedGradient.end(), minimizedbackgroundColor );

		minimizedbevelSize = GetxRCInt(name, "MinimizedButtonSolidBevelSize", normalbevelSize, 0);

		parseGradientCFG(GetxRCLine(name, "MinimizedButtonSolidGradientColors", normalgradientColor), 2);

		minimizedverticalGradient = GetxRCBool(name, "MinimizedButtonSolidGradientVertical", normalverticalGradient);
	}
	else
	{
		imageFile = GetxRCString(name, "MinimizedButtonImage", "");
		if (!imageFile.empty())
		{
			memoryMinimizedBitmap = LoadLSImage(imageFile.c_str(), 0);

			/*if (memoryMinimizedBitmap)
			{
				BITMAP bm;
				GetObject(memoryMinimizedBitmap, sizeof(BITMAP), &bm);
				has_minimizedAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_minimizedAlphaMap)
					PrepareAlphaChannel(memoryMinimizedBitmap);
			}*/

			GetLSBitmapSize(memoryMinimizedBitmap, &imageMinimizedWidth, &imageMinimizedHeight);

			memoryMinimizedDC = CreateCompatibleDC(0);
			SelectObject(memoryMinimizedDC, memoryMinimizedBitmap);
		}
	}

	useflashingSolidColors = GetxRCBool(name, "UseFlashingButtonSolidColors", usenormalSolidColors);

	if (useflashingSolidColors)
	{
		imageFile = GetxRCLine(name, "FlashingButtonSolidColors", "ffffff");
		parseCFG(imageFile, flashingbackgroundColor, flashinglightBevelColor, flashingdarkBevelColor);
		//BG Insert in Gradient List
		FlashingGradient.insert(FlashingGradient.end(), flashingbackgroundColor );

		flashingbevelSize = GetxRCInt(name, "FlashingButtonSolidBevelSize", normalbevelSize, 0);
	
		parseGradientCFG(GetxRCLine(name, "FlashingButtonSolidGradientColors", normalgradientColor), 3);

		flashingverticalGradient = GetxRCBool(name, "FlashingButtonSolidGradientVertical", normalverticalGradient);
	}
	else
	{
		imageFile = GetxRCString(name, "FlashingButtonImage", "");
		if (!imageFile.empty())
		{
			memoryFlashingBitmap = LoadLSImage(imageFile.c_str(), 0);

			/*if (memoryFlashingBitmap)
			{
				BITMAP bm;
				GetObject(memoryFlashingBitmap, sizeof(BITMAP), &bm);
				has_flashingAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_flashingAlphaMap)
					PrepareAlphaChannel(memoryFlashingBitmap);
			}*/

			GetLSBitmapSize(memoryFlashingBitmap, &imageFlashingWidth, &imageFlashingHeight);

			memoryFlashingDC = CreateCompatibleDC(0);
			SelectObject(memoryFlashingDC, memoryFlashingBitmap);
		}
	}

	usehoverSolidColors = GetxRCBool(name, "UseHoverButtonSolidColors", usenormalSolidColors);

	if (usehoverSolidColors)
	{
		imageFile = GetxRCLine(name, "HoverButtonSolidColors", "ffffff");
		parseCFG(imageFile, hoverbackgroundColor, hoverlightBevelColor, hoverdarkBevelColor);
		//BG Insert in Gradient List
		HoverGradient.insert(HoverGradient.end(), hoverbackgroundColor );

		hoverbevelSize = GetxRCInt(name, "HoverButtonSolidBevelSize", normalbevelSize, 0);

		parseGradientCFG(GetxRCLine(name, "HoverButtonSolidGradientColors", normalgradientColor), 4);

		hoververticalGradient = GetxRCBool(name, "HoverButtonSolidGradientVertical", normalverticalGradient);
	}
	else
	{
		imageFile = GetxRCString(name, "HoverButtonImage", "");
		if (!imageFile.empty())
		{
			memoryHoverBitmap = LoadLSImage(imageFile.c_str(), 0);

			/*if (memoryHoverBitmap)
			{
				BITMAP bm;
				GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);
				has_hoverAlphaMap = (bm.bmBits != 0) && (bm.bmBitsPixel == 32);

				if (has_hoverAlphaMap)
					PrepareAlphaChannel(memoryHoverBitmap);
			}*/

			GetLSBitmapSize(memoryHoverBitmap, &imageHoverWidth, &imageHoverHeight);

			memoryHoverDC = CreateCompatibleDC(0);
			SelectObject(memoryHoverDC, memoryHoverBitmap);
		}
	}

	normalbutton_saturation = GetxRCInt(name , "NormalButtonSaturationIntensity", defaultbutton_saturation, 0, 255);
	normalbutton_hueIntensity = GetxRCInt(name , "NormalButtonHueIntensity", defaultbutton_hueIntensity, 0, 255);
	normalbutton_hueColor = GetxRCColor(name, "NormalButtonHueColor", defaultbutton_hueColor);

	normalAlpha = GetxRCInt(name , "NormalButtonAlphaBlend", defaultAlpha, 0, 255);

	normalleftEdge = GetxRCInt(name, "NormalButtonImageLeftEdge", defaultleftEdge, 0);
	normaltopEdge = GetxRCInt(name, "NormalButtonImageTopEdge", defaulttopEdge, 0);
	normalrightEdge = GetxRCInt(name, "NormalButtonImageRightEdge", defaultrightEdge, 0);
	normalbottomEdge = GetxRCInt(name, "NormalButtonImageBottomEdge", defaultbottomEdge, 0);

	normalmode = GetxRCNamedValue(name, "NormalButtonImageMode", modeConstants, defaultmode);

	activebutton_saturation = GetxRCInt(name , "ActiveButtonSaturationIntensity", normalbutton_saturation, 0, 255);
	activebutton_hueIntensity = GetxRCInt(name , "ActiveButtonHueIntensity", normalbutton_hueIntensity, 0, 255);
	activebutton_hueColor = GetxRCColor(name, "ActiveButtonHueColor", normalbutton_hueColor);

	activeAlpha = GetxRCInt(name , "ActiveButtonAlphaBlend", normalAlpha, 0, 255);

	activeleftEdge = GetxRCInt(name, "ActiveButtonImageLeftEdge", normalleftEdge, 0);
	activetopEdge = GetxRCInt(name, "ActiveButtonImageTopEdge", normaltopEdge, 0);
	activerightEdge = GetxRCInt(name, "ActiveButtonImageRightEdge", normalrightEdge, 0);
	activebottomEdge = GetxRCInt(name, "ActiveButtonImageBottomEdge", normalbottomEdge, 0);

	activemode = GetxRCNamedValue(name, "ActiveButtonImageMode", modeConstants, normalmode);

	minimizedbutton_saturation = GetxRCInt(name , "MinimizedButtonSaturationIntensity", normalbutton_saturation, 0, 255);
	minimizedbutton_hueIntensity = GetxRCInt(name , "MinimizedButtonHueIntensity", normalbutton_hueIntensity, 0, 255);
	minimizedbutton_hueColor = GetxRCColor(name, "MinimizedButtonHueColor", normalbutton_hueColor);

	minimizedAlpha = GetxRCInt(name , "MinimizedButtonAlphaBlend", normalAlpha, 0, 255);

	minimizedleftEdge = GetxRCInt(name, "MinimizedButtonImageLeftEdge", normalleftEdge, 0);
	minimizedtopEdge = GetxRCInt(name, "MinimizedButtonImageTopEdge", normaltopEdge, 0);
	minimizedrightEdge = GetxRCInt(name, "MinimizedButtonImageRightEdge", normalrightEdge, 0);
	minimizedbottomEdge = GetxRCInt(name, "MinimizedButtonImageBottomEdge", normalbottomEdge, 0);

	minimizedmode = GetxRCNamedValue(name, "MinimizedButtonImageMode", modeConstants, normalmode);

	flashingbutton_saturation = GetxRCInt(name , "FlashingButtonSaturationIntensity", normalbutton_saturation, 0, 255);
	flashingbutton_hueIntensity = GetxRCInt(name , "FlashingButtonHueIntensity", normalbutton_hueIntensity, 0, 255);
	flashingbutton_hueColor = GetxRCColor(name, "FlashingButtonHueColor", normalbutton_hueColor);

	flashingAlpha = GetxRCInt(name , "FlashingButtonAlphaBlend", normalAlpha, 0, 255);

	flashingleftEdge = GetxRCInt(name, "FlashingButtonImageLeftEdge", normalleftEdge, 0);
	flashingtopEdge = GetxRCInt(name, "FlashingButtonImageTopEdge", normaltopEdge, 0);
	flashingrightEdge = GetxRCInt(name, "FlashingButtonImageRightEdge", normalrightEdge, 0);
	flashingbottomEdge = GetxRCInt(name, "FlashingButtonImageBottomEdge", normalbottomEdge, 0);

	flashingmode = GetxRCNamedValue(name, "FlashingButtonImageMode", modeConstants, normalmode);

	hoverbutton_saturation = GetxRCInt(name , "HoverButtonSaturationIntensity", normalbutton_saturation, 0, 255);
	hoverbutton_hueIntensity = GetxRCInt(name , "HoverButtonHueIntensity", normalbutton_hueIntensity, 0, 255);
	hoverbutton_hueColor = GetxRCColor(name, "HoverButtonHueColor", normalbutton_hueColor);

	hoverAlpha = GetxRCInt(name , "HoverButtonAlphaBlend", normalAlpha, 0, 255);

	hoverleftEdge = GetxRCInt(name, "HoverButtonImageLeftEdge", normalleftEdge, 0);
	hovertopEdge = GetxRCInt(name, "HoverButtonImageTopEdge", normaltopEdge, 0);
	hoverrightEdge = GetxRCInt(name, "HoverButtonImageRightEdge", normalrightEdge, 0);
	hoverbottomEdge = GetxRCInt(name, "HoverButtonImageBottomEdge", normalbottomEdge, 0);

	hovermode = GetxRCNamedValue(name, "HoverButtonImageMode", modeConstants, normalmode);

	if ((normalbutton_saturation != 255 || normalbutton_hueIntensity != 0) /*&& !has_normalAlphaMap*/ && !usenormalSolidColors)
		paintEffects(memoryNormalDC, imageNormalWidth, imageNormalHeight, normalbutton_saturation, normalbutton_hueIntensity, normalbutton_hueColor);
	if (memoryActiveDC && (activebutton_saturation != 255 || activebutton_hueIntensity != 0) /*&& !has_activeAlphaMap*/ && !useactiveSolidColors)
		paintEffects(memoryActiveDC, imageActiveWidth, imageActiveHeight, activebutton_saturation, activebutton_hueIntensity, activebutton_hueColor);
	if (memoryMinimizedDC && (minimizedbutton_saturation != 255 || minimizedbutton_hueIntensity != 0) /*&& !has_minimizedAlphaMap*/ && !useminimizedSolidColors)
		paintEffects(memoryMinimizedDC, imageMinimizedWidth, imageMinimizedHeight, minimizedbutton_saturation, minimizedbutton_hueIntensity, minimizedbutton_hueColor);
	if (memoryFlashingDC && (flashingbutton_saturation != 255 || flashingbutton_hueIntensity != 0) /*&& !has_flashingAlphaMap*/ && !useflashingSolidColors)
		paintEffects(memoryFlashingDC, imageFlashingWidth, imageFlashingHeight, flashingbutton_saturation, flashingbutton_hueIntensity, flashingbutton_hueColor);
	if (memoryHoverDC && (hoverbutton_saturation != 255 || hoverbutton_hueIntensity != 0) /*&& !has_hoverAlphaMap*/ && !usehoverSolidColors)
		paintEffects(memoryHoverDC, imageHoverWidth, imageHoverHeight, hoverbutton_saturation, hoverbutton_hueIntensity, hoverbutton_hueColor);
}

void ButtonTexture::apply(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	RECT r;
	r.left = x;
	r.top = y;
	r.right = r.left + width;
	r.bottom = r.top + height;

	if (buttonType == 0 && usenormalSolidColors)
	{
		// fill in the background
		if (normalbevelSize > 0 && (normalbevelSize*2 < width && normalbevelSize*2 < height))
		{
			::SetBkColor(hDC, normaldarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-normalbevelSize;
			r.bottom = r.bottom-normalbevelSize;
			::SetBkColor(hDC, normallightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+normalbevelSize;
			r.top = r.top+normalbevelSize;

			if ( NormalGradient.size() > 1 )
				solidGradientPaint(hDC, NormalGradient, r, normalverticalGradient);
			else
			{
				::SetBkColor(hDC, normalbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( NormalGradient.size() > 1 )
				solidGradientPaint(hDC, NormalGradient, r, normalverticalGradient);
			else
			{
				::SetBkColor(hDC, normalbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 1 && useactiveSolidColors )
	{
		// fill in the background
		if (activebevelSize > 0 && (activebevelSize*2 < width && activebevelSize*2 < height))
		{
			::SetBkColor(hDC, activedarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-activebevelSize;
			r.bottom = r.bottom-activebevelSize;
			::SetBkColor(hDC, activelightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+activebevelSize;
			r.top = r.top+activebevelSize;

			if ( ActiveGradient.size() > 1 )
				solidGradientPaint(hDC, ActiveGradient, r, activeverticalGradient);
			else
			{
				::SetBkColor(hDC, activebackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( ActiveGradient.size() > 1 )
				solidGradientPaint(hDC, ActiveGradient, r, activeverticalGradient);
			else
			{
				::SetBkColor(hDC, activebackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && useminimizedSolidColors )
	{
		// fill in the background
		if (minimizedbevelSize > 0 && (minimizedbevelSize*2 < width && minimizedbevelSize*2 < height))
		{
			::SetBkColor(hDC, minimizeddarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-minimizedbevelSize;
			r.bottom = r.bottom-minimizedbevelSize;
			::SetBkColor(hDC, minimizedlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+minimizedbevelSize;
			r.top = r.top+minimizedbevelSize;

			if ( MinimizedGradient.size() > 1 )
				solidGradientPaint(hDC, MinimizedGradient, r, minimizedverticalGradient);
			else
			{
				::SetBkColor(hDC, minimizedbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( MinimizedGradient.size() > 1 )
				solidGradientPaint(hDC, MinimizedGradient, r, minimizedverticalGradient);
			else
			{
				::SetBkColor(hDC, minimizedbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 3 && useflashingSolidColors )
	{
		// fill in the background
		if (flashingbevelSize > 0 && (flashingbevelSize*2 < width && flashingbevelSize*2 < height))
		{
			::SetBkColor(hDC, flashingdarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-flashingbevelSize;
			r.bottom = r.bottom-flashingbevelSize;
			::SetBkColor(hDC, flashinglightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+flashingbevelSize;
			r.top = r.top+flashingbevelSize;

			if ( FlashingGradient.size() > 1 )
				solidGradientPaint(hDC, FlashingGradient, r, flashingverticalGradient);
			else
			{
				::SetBkColor(hDC, flashingbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FlashingGradient.size() > 1 )
				solidGradientPaint(hDC, FlashingGradient, r, flashingverticalGradient);
			else
			{
				::SetBkColor(hDC, flashingbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 4 && usehoverSolidColors )
	{
		// fill in the background
		if (hoverbevelSize > 0 && (hoverbevelSize*2 < width && hoverbevelSize*2 < height))
		{
			::SetBkColor(hDC, hoverdarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hoverbevelSize;
			r.bottom = r.bottom-hoverbevelSize;
			::SetBkColor(hDC, hoverlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hoverbevelSize;
			r.top = r.top+hoverbevelSize;

			if ( HoverGradient.size() > 1 )
				solidGradientPaint(hDC, HoverGradient, r, hoververticalGradient);
			else
			{
				::SetBkColor(hDC, hoverbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
				solidGradientPaint(hDC, HoverGradient, r, hoververticalGradient);
			else
			{
				::SetBkColor(hDC, hoverbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if (buttonType == 4 && memoryHoverBitmap == 0)
			return;
	
		if ( buttonType == 1 && memoryActiveBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 2 && memoryMinimizedBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
			buttonType = 0;

		if (buttonType == 0 && memoryNormalBitmap == 0)
			return;
		
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(bufferDC, bufferBitmap);

		if (buttonType == 0)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
					memoryNormalDC, 0, 0, imageNormalWidth, imageNormalHeight,
					normalleftEdge, normaltopEdge, normalrightEdge, normalbottomEdge,
					normalmode, SRCCOPY);
		}
		else if (buttonType == 1)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
					memoryActiveDC, 0, 0, imageActiveWidth, imageActiveHeight,
					activeleftEdge, activetopEdge, activerightEdge, activebottomEdge,
					activemode, SRCCOPY);
		}
		else if (buttonType == 2)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
					memoryMinimizedDC, 0, 0, imageMinimizedWidth, imageMinimizedHeight,
					minimizedleftEdge, minimizedtopEdge, minimizedrightEdge, minimizedbottomEdge,
					minimizedmode, SRCCOPY);
		}
		else if (buttonType == 3)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFlashingDC, 0, 0, imageFlashingWidth, imageFlashingHeight,
					flashingleftEdge, flashingtopEdge, flashingrightEdge, flashingbottomEdge,
					flashingmode, SRCCOPY);
		}
		else if (buttonType == 4)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, imageHoverWidth, imageHoverHeight,
					hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
					hovermode, SRCCOPY);
		}
			
		TransparentBltLS(hDC, x, y, width, height,
					bufferDC, 0, 0,
					RGB(255, 0, 255));

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
}

void ButtonTexture::paintEffects(HDC destHDC, int width, int height, int button_saturation, int button_hueIntensity, COLORREF button_hueColor)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
		
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				// Saturation effect
				if(button_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
					g = (BYTE)((g * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
					b = (BYTE)((b * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
				}

				// Hue effect
				if(button_hueIntensity != 0)
				{
					// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
					// test normal
					r = (BYTE)((r * (255 - button_hueIntensity) + GetRValue(button_hueColor) * button_hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - button_hueIntensity) + GetGValue(button_hueColor) * button_hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - button_hueIntensity) + GetBValue(button_hueColor) * button_hueIntensity + 255) >> 8);
				}
			}

			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void ButtonTexture::solidGradientPaint(HDC destHDC, vector<COLORREF> workGradient, RECT r, bool vertical)
{
	int nWidth = r.right - r.left;
	int nHeight = r.bottom - r.top;

	RECT r2;
	double steps = 1;
	double cr, cg, cb, dr, dg, db;
	int gradSteps;

	int colorCount = workGradient.size();

	if (vertical)
		gradSteps = max(nHeight / (colorCount-1), 1);
	else
		gradSteps = max(nWidth / (colorCount-1), 1);

	int lastGradientPos;
	if (vertical)
		lastGradientPos = r.top;
	else
		lastGradientPos = r.left;

	for (ColorRefIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
	{
		if (cit == workGradient.end())
			break;

		COLORREF one = *cit;
		cit++;
		COLORREF two = *cit;
		cit--;
	
		// get current color
		cr = GetRValue( one );
		cg = GetGValue( one );
		cb = GetBValue( one );

		// Get delta color
		dr = (cr - GetRValue( two )) / gradSteps;
		dg = (cg - GetGValue( two )) / gradSteps;
		db = (cb - GetBValue( two )) / gradSteps;

		for (int i = 1; i <= gradSteps; i++)
		{
			cr -= dr;
			cg -= dg;
			cb -= db;
		
			if (vertical)
				SetRect(&r2, r.left, (int)(lastGradientPos+steps*(i - 1)), r.right, min((int)(lastGradientPos+steps*i), r.bottom));
			else
				SetRect(&r2, (int)(lastGradientPos+steps*(i - 1)), r.top, min((int)(lastGradientPos+steps*i), r.right), r.bottom);

			if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
			{
				::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
				::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
			}
		}
		if (vertical)
			lastGradientPos = r2.bottom;
		else
			lastGradientPos = r2.right;
	}
}

/*void ButtonTexture::PrepareAlphaChannel(HBITMAP &bmp)
{
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//PreMultiply
		pPixel[0] = pPixel[0] * pPixel[3] / 255;
		pPixel[1] = pPixel[1] * pPixel[3] / 255;
		pPixel[2] = pPixel[2] * pPixel[3] / 255;

		pPixel+= 4;
	}
}*/