#include "stdafx.h"
#include "Taskbar.h"
#include "TaskbarButton.h"
#include "TaskbarTexture.h"
#include "ButtonTexture.h"
#include "ButtonIcon.h"
#include "ButtonFont.h"

#include "DropTarget.h"
#include "WindowUtilities.h"

#include "shlobj.h"

#define LM_UPDATEBG (WM_USER + 1)

#define GET_X_LPARAM(lp) ((int)(short)LOWORD(lp))
#define GET_Y_LPARAM(lp) ((int)(short)HIWORD(lp))

//hack!!
int nButtonsAddedAtStartup;

vector<string> RemoveList;
vector<string> CustomIconList;
vector<string> SpecificList;

// undocumented SwitchToThisWindow API
void (WINAPI *SwitchToThisWindow)(HWND, int);

//Alpha Transparency
typedef DWORD (WINAPI *PSLWA)(HWND, DWORD, BYTE, DWORD);
PSLWA pSetLayeredWindowAttributes = NULL;

#define TIMER_UPDATE		1
#define TIMER_MOUSETRACK	2

UINT lsMessages[] = {
                      LM_LSSELECT,
                      LM_REDRAW,
                      LM_WINDOWACTIVATED,
                      LM_WINDOWCREATED,
                      LM_WINDOWDESTROYED,
					  LM_REFRESH,
					  LM_RECYCLE,
                      0
					 };


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Taskbar::Taskbar(const string &name):actualx(0),actualy(0),actualwidth(0),actualheight(0)
{
	this->name = name;

	hWnd = 0;
	hInstance = 0;
	box = 0;

	backgroundDC = 0;
	bufferDC = 0;
	backgroundBitmap = 0;
	bufferBitmap = 0;

	alphaInitialized = false;
	layeredFlag = false;

	transparencyNeeded = true;

	mouseInside = false;
	focusOnEnter = 0;
	//pressedModifierKey = 0;

	visible = false;

	inCriticalProcess = false;
	autolines_rows = 0;

	useGrouping = false;

	useHoverState = false;

	taskbartexture = new TaskbarTexture();
	buttontexture = new ButtonTexture(name);
	buttonicon = new ButtonIcon(name);
	buttonfont = new ButtonFont(name);

	movecapture = NULL;
	capture = NULL;

	SwitchToThisWindow = (void (WINAPI *)(HWND, int)) 
							GetProcAddress(GetModuleHandle("USER32.DLL"), "SwitchToThisWindow");

	pDropTarget = NULL;
}

Taskbar::~Taskbar()
{
	RemoveList.clear();
	CustomIconList.clear();
	SpecificList.clear();

	if (pDropTarget)
	{
		pDropTarget->Revoke( hWnd );
		delete pDropTarget;
	}

	if (hWnd && ToolTips)
	{
		updateHint(hWnd,"", false, true);
		DestroyWindow(ToolTips);
		ToolTips = 0;
	}

	// shutdown timer
	if(IsWindow(hWnd))
	{
		KillTimer(hWnd, TIMER_UPDATE);
		KillTimer(hWnd, TIMER_MOUSETRACK);
	}

	// release buttons
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{	
		delete (*tbit);
		*tbit = NULL;
	}
	taskbuttons.clear();

	if(IsWindow(hWnd))
	{
		DestroyWindow(hWnd);
	}

	if(backgroundBitmap)
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
		DeleteObject(backgroundBitmap);
	}
	if (backgroundDC)
		DeleteDC(backgroundDC);

	if(bufferBitmap)
	{
		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
	}
	if (bufferDC)
		DeleteDC(bufferDC);

	if (buttontexture)
		delete buttontexture;
	if (buttonicon)
		delete buttonicon;
	if (buttonfont)
		delete buttonfont;
	if (taskbartexture)
		delete taskbartexture;

	// unregister messages
	SendMessage( GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::lsMessages );
}

void Taskbar::load(HINSTANCE hInstance, HWND box)
{
	this->hInstance = hInstance;
	
	if (IsWindow(box))
	{
		this->box = box;
	}
	else
	{
		this->box = NULL;
	}
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	hWnd = CreateWindowEx(box ? 0 : WS_EX_TOOLWINDOW,
		"xTaskbar",
		name.c_str(),
		(box ? WS_CHILD : WS_POPUP),
		actualx, actualy,		
		actualwidth, actualheight,
		box ? box : GetLitestepWnd(), 
		0,
		hInstance,
		this);
		
	if (hWnd)
	{
		SendMessage( GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::lsMessages );
		
		SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

		reconfigure();

		if( nButtonsAddedAtStartup > 0 )
		{
			for(int nI = 0; nI < nButtonsAddedAtStartup; ++nI)
			{
				if(!taskbarAdd.empty())
				{
					LSExecute(hWnd, taskbarAdd.c_str(), SW_SHOWNORMAL);
				}
			}
			nButtonsAddedAtStartup = 0;
		}

		//we become a drop target
		pDropTarget = new DropTarget;
		pDropTarget->Register( hWnd );
		pDropTarget->RegisterDrop( hWnd, MOUSEISDRAGGING );
	}
	else
		delete this;
}

void Taskbar::reconfigure(bool refresh)
{
	nButtonsAddedAtStartup = 0;

	NameValuePair buttonActionValues[] = {
			{ ".none", 0 },
			{ ".select", 1 },
			{ ".menu", 2 },
			{ ".close", 3 },
			{ ".ontop", 4 },
			{ 0, 0 }
			};

	NameValuePair directionValues[] = {
			{ ".right", 1 },
			{ ".left", 2 },
			{ ".bottom", 3 },
			{ ".top", 4 },
			{ 0, 0 }
			};

	NameValuePair displayValues[] = {
			{ "all", -1 },
			{ "normal+active+minimized", -1 },
			{ "normal", 0 },
			{ "active", 1 },
			{ "minimized", 2 },
			{ "active+normal", 3 },
			{ "normal+active", 3 },
			{ "minimized+normal", 4 },
			{ "normal+minimized", 4 },
			{ "active+minimized", 5 },
			{ "minimized+active", 5 },
			{ 0, 0 }
			};

	if (refresh)
	{
		RemoveList.clear();
		CustomIconList.clear();
		SpecificList.clear();
	}

	GetxRCList("*xTaskbarRemove", RemoveList);
	GetxRCList("*xTaskbarIcon", CustomIconList);
	GetxRCList("*xTaskbarSpecific", SpecificList);

	displayButtonType = GetxRCNamedValue(name, "Display", displayValues, -1);

	lButtonAction = GetxRCNamedValue(name, "OnLeftClickButton", buttonActionValues, 1);
	rButtonAction = GetxRCNamedValue(name, "OnRightClickButton", buttonActionValues, 2);
	mButtonAction = GetxRCNamedValue(name, "OnMiddleClickButton", buttonActionValues, 0);

	useGrouping = GetxRCBool(name, "UseGrouping");

	useBigIcon = GetxRCBool(name, "UseBigIcon");
	//if (useGrouping)
	//	minimizeToTray = false;
	//else
	//	minimizeToTray = GetxRCBool(name, "MinimizeToTray");

	noMinimizeOnClick = GetxRCBool(name, "NoMinimizeOnClick");

	//Events/Actions
	taskbarLButtonUp = GetxRCLine(name, "OnLeftClickUp", "");
	taskbarLButtonDown =GetxRCLine(name, "OnLeftClickDown", "");
	taskbarMButtonUp = GetxRCLine(name, "OnMiddleClickUp", "");
	taskbarMButtonDown = GetxRCLine(name, "OnMiddleClickDown", "");
	taskbarRButtonUp = GetxRCLine(name, "OnRightClickUp", "");
	taskbarRButtonDown = GetxRCLine(name, "OnRightClickDown", "");
	taskbarMWheelDown = GetxRCLine(name, "OnWheelDown", "");
	taskbarMWheelUp = GetxRCLine(name, "OnWheelUp", "");
	taskbarEnter = GetxRCLine(name, "OnEnter", "");
	taskbarLeave = GetxRCLine(name, "OnLeave", "");
	taskbarMove = GetxRCLine(name, "OnMove", "");
	taskbarResize = GetxRCLine(name, "OnResize", "");
	taskbarAdd = GetxRCLine(name, "OnAdd", "");
	taskbarRemove = GetxRCLine(name, "OnRemove", "");

	taskbarLineAdd = GetxRCLine(name, "OnLineAdd", "");
	taskbarLineRemove = GetxRCLine(name, "OnLineRemove", "");

	showIcon = GetxRCBool(name, "ShowIcon", true);
	showText = GetxRCBool(name, "ShowText", true);
	showToolTip = GetxRCBool(name, "ShowTooltip", true);

	useHoverState = GetxRCBool(name, "ShowHoverState");

	loadInBox = GetxRCBool(name, "LoadInBox");
	startHidden = GetxRCBool(name, "StartHidden");
	transparencyMode = GetxRCBool(name, "TransparencyMode");
	focusOnEnter = GetxRCBool(name, "FocusOnEnter");

	leftBorder = GetxRCInt(name, "LeftBorder", 0, 0);
	topBorder = GetxRCInt(name, "TopBorder", 0, 0);
	rightBorder = GetxRCInt(name, "RightBorder", 0, 0);
	bottomBorder = GetxRCInt(name, "BottomBorder", 0, 0);

	if (!refresh)
	{
		actualwidth = GetxRCDimen(name, "Width", GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CXSCREEN));
		actualheight = GetxRCDimen(name, "Height", 32, GetSystemMetrics(SM_CYSCREEN));
	}
	lines_rows = GetxRCInt(name, "Lines", 1, 1);
	maxlines_rows = GetxRCInt(name, "MaxLines", 100, 1);
	if (lines_rows > maxlines_rows)
		lines_rows = maxlines_rows;

	direction = GetxRCNamedValue(name, "ButtonDirection", directionValues, 1);

	maxTaskWidth = GetxRCInt(name, "MaxButtonWidth", actualwidth-leftBorder-rightBorder);
	maxTaskHeight = GetxRCInt(name, "MaxButtonHeight", actualheight-topBorder-bottomBorder);
	minTaskWidth = GetxRCInt(name, "MinButtonWidth", 0);
	minTaskHeight = GetxRCInt(name, "MinButtonHeight", 0);
	xSpacing = GetxRCInt(name, "ButtonxSpacing", 0, 0);
	ySpacing = GetxRCInt(name, "ButtonySpacing", 0, 0);

	autoNewLine = GetxRCBool(name, "AutoLines");
	
	if (refresh)
	{
		if (buttontexture)
			delete buttontexture;
		if (buttonicon)
			delete buttonicon;
		if (buttonfont)
			delete buttonfont;
		if (taskbartexture)
			delete taskbartexture;

		taskbartexture = new TaskbarTexture();
		buttontexture = new ButtonTexture(name);
		buttonicon = new ButtonIcon(name);
		buttonfont = new ButtonFont(name);
	}
	taskbartexture->configure(name);
	buttontexture->configure(name);
	buttonicon->configure(name);
	buttonfont->configure(name);

	transparencyNeeded = taskbartexture->isTransparencyNeeded();

	timerRefresh = GetxRCInt(name, "UpdateInterval", 1000, 100);
	flashRefresh = GetxRCInt(name, "FlashInterval", timerRefresh, 100);

	if (!refresh)
	{
		if (box == NULL)
		{
			actualx = GetxRCCoord(name, "X", 0, GetSystemMetrics(SM_CXSCREEN));
			actualy = GetxRCCoord(name, "Y", 0, GetSystemMetrics(SM_CYSCREEN));

			reposition(actualx, actualy, actualwidth, actualheight);
		}
		else
		{
			RECT rc;
			GetClientRect(box, &rc);

			actualx = GetxRCCoord(name, "X", 0, rc.right-rc.left);
			actualy = GetxRCCoord(name, "Y", 0, rc.bottom-rc.top);

			reposition(actualx, actualy, actualwidth, actualheight);
		}
	}

	EnumWindows(Taskbar::EnumWindowsProc, (LPARAM) this);

	CreateLayout();

	if (!refresh)
	{
		ToolTips = CreateWindow(TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP,
								0, 0, 0, 0,
								NULL, NULL, hInstance, NULL);

		SetWindowPos( ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
		
		if (ToolTips != 0)
		{
			TOOLINFO ti;
			RECT clientRect;

			GetClientRect(hWnd, &clientRect);
			ti.cbSize = sizeof(TOOLINFO);
			ti.uFlags = TTF_SUBCLASS;
			ti.hwnd = hWnd;
			ti.uId = 0;
			ti.rect = clientRect;
			ti.hinst = hInstance;
			ti.lpszText = 0;
			ti.lParam = 0;

			SendMessage(ToolTips, TTM_ADDTOOL, 0, LPARAM(&ti));
		}

		updateHint(hWnd, "", true);
	}

	alphaTransparency = GetxRCInt(name, "AlphaTransparency", 255, 0, 255);

	if ( IsOS(OS_2KXP) && alphaTransparency != 255 )
	{
		if (!refresh)
		{
			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			UpdateWindowTransparent(hWnd, alphaTransparency);
		}
	}

	if (!refresh)
	{
		setAlwaysOnTop(GetxRCBool(name, "AlwaysOnTop"));

		if (startHidden || loadInBox)
			hide();
		else
			show();
	}

	repaint(true);

	SetTimer(hWnd, TIMER_UPDATE, timerRefresh, NULL);
}

void Taskbar::SwitchToButton( LPCSTR direction )
{
	HWND current = NULL;
	HWND prev = NULL;
	HWND next = NULL;

	current = lastActive;

	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if((*tbit)->hButtonTask == current)
		{
			++tbit;
			if(tbit != taskbuttons.end())
				next = (*tbit)->hButtonTask ;		
		}
		else
		{
			prev = (*tbit)->hButtonTask ;
			++tbit;
		}
	}

	if( stricmp(direction, "next") == NULL && next != NULL)
		SelectWindow(next);
	else if ( stricmp(direction, "prev") == NULL && prev != NULL)
		SelectWindow(prev);
}
	
//------------------------------------------------------------
//Set Parent Box (nearly never needed/called!)
//------------------------------------------------------------
void Taskbar::setBox(HWND newparent)
{
	if (box == newparent)
		return;

	if (alwaysOnTop)
		setAlwaysOnTop(FALSE);
	
	box = newparent;

	ModifyStyleEx(hWnd, WS_EX_LAYERED, 0);
	
	ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
	SetParent(hWnd, newparent);
}

void Taskbar::setAlwaysOnTop(bool alwaysOnTop)
{
	if (box)
		return;
	
	this->alwaysOnTop = alwaysOnTop;
	
	if(hWnd)
	{
		ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
		ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
		
		SetWindowPos(hWnd, alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			0, 0, 0, 0,
			SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
}

void Taskbar::setAlpha(int newAlphaTransparency)
{
	if (box)
		return;

	if ( newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!layeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			layeredFlag = true;
		}

		alphaTransparency = newAlphaTransparency;

		UpdateWindowTransparent(hWnd, newAlphaTransparency);
	}
}

BOOL Taskbar::UpdateWindowTransparent(HWND hWnd, int factor)
{
	/* First, see if we can get the API call we need. If we've tried
    * once, we don't need to try again. */
	if (!alphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pSetLayeredWindowAttributes = 
					(PSLWA) GetProcAddress(hDLL, "SetLayeredWindowAttributes");

		alphaInitialized = TRUE;
   
		if (pSetLayeredWindowAttributes == NULL)
		{
		    ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			layeredFlag = false;
			return FALSE;
		}
	}

	/* Now, we need to set the 'layered window attributes'. This
	 * is where the alpha values get set. */
  
	return pSetLayeredWindowAttributes (hWnd, 0, factor, LWA_ALPHA);
}

//------------------------------------------------------------
//Show/Hide
//------------------------------------------------------------
void Taskbar::show()
{
	if(hWnd)
	{
		ShowWindow(hWnd, SW_SHOWNOACTIVATE);

		visible = true;
	}
}

void Taskbar::hide()
{
	if(hWnd)
	{
		ShowWindow(hWnd, SW_HIDE);

		visible = false;
	}
}

//------------------------------------------------------------
//Move/Resize/Reposition
//------------------------------------------------------------
void Taskbar::move(int newx, int newy, int steps, int time, bool moveBy)
{
	if ( steps > 0 && time > 0 )
	{
		if (time > 50)
			time = 50;

		if (moveBy)
		{
			newx = actualx + newx;
			newy = actualy + newy;
		}

		int counter = 0;
		int deltax = newx - actualx;
		int deltay = newy - actualy;

		bool growx;
		bool growy;
		if (deltax < 0)
			growx = false;
		else
			growx = true;

		if (deltay < 0)
			growy = false;
		else
			growy = true;

		div_t div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;

		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		int dynx = actualx;
		int dyny = actualy;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0; // Stop if x and y finished, stopper = 2
				dynx = dynx+deltax;
				if (growx && dynx > newx)
				{
					dynx = newx;
					stopper++;
				}
				else if (!growx && dynx < newx)
				{
					dynx = newx;
					stopper++;
				}

				dyny = dyny+deltay;
				if (growy && dyny > newy)
				{
					dyny = newy;
					stopper++;
				}
				else if (!growy && dyny < newy)
				{
					dyny = newy;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, dynx, dyny, 0, 0,
					SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);

				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (stopper == 2)
					counter = steps;
				
				counter++;

				Sleep(time);
			}

			actualx = newx;
			actualy = newy;

			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);
		}
	}
	else
	{
		if(hWnd)
		{
			if (moveBy)
			{
				newx = actualx + newx;
				newy = actualy + newy;
			}

			actualx = newx;
			actualy = newy;

			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
				SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}

	if(!taskbarMove.empty())
		LSExecute(hWnd, taskbarMove.c_str(), SW_SHOWNORMAL);
}

void Taskbar::resize(int newwidth, int newheight, int steps, int time, bool resizeBy)
{
	if ( steps > 0 && time > 0 )
	{
		if (time > 50)
			time = 50;

		if (resizeBy)
		{
			newwidth = actualwidth + newwidth;
			newheight = actualheight + newheight;
		}

		int counter = 0;
		int deltax = newwidth - actualwidth;
		int deltay = newheight - actualheight;

		bool growx;
		bool growy;
		if (deltax < 0)
			growx = false;
		else
			growx = true;

		if (deltay < 0)
			growy = false;
		else
			growy = true;

		div_t div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		int dynwidth = actualwidth;
		int dynheight = actualheight;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0;  // Stop if width and height are finished, stopper = 2
				dynwidth = dynwidth+deltax;
				if (growx && dynwidth > newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}
				else if (!growx && dynwidth < newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}

				dynheight = dynheight+deltay;
				if (growy && dynheight > newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				else if (!growy && dynheight < newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, 0, 0, dynwidth, dynheight,
					SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
				
				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				CreateLayout();

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (stopper == 2)
					counter = steps;

				counter++;

				Sleep(time);
			}

			actualheight = newheight;
			actualwidth = newwidth;

			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
		
			//Force Repaint to get correct values for Resize Command (SetEvar must be called befire!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}
	else
	{
		if(hWnd)
		{
			if (resizeBy)
			{
				newwidth = actualwidth + newwidth;
				newheight = actualheight + newheight;
			}

			actualheight = newheight;
			actualwidth = newwidth;

			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
				SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called before!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}

	//updateRegion to track Magic Pink Changes is inside Createlayout()
	CreateLayout();

	if(!taskbarResize.empty())
		LSExecute(hWnd, taskbarResize.c_str(), SW_SHOWNORMAL);

	//Set new Rect for ToolTip Tracker
	if (showToolTip && ToolTips)
		updateHint(hWnd, "", true);
}

void Taskbar::reposition(int newx, int newy, int newwidth, int newheight, int steps, int time, bool repositionBy)
{
	if (steps > 0 && time > 0)
	{
		if (time > 50)
			time = 50;

		if (repositionBy)
		{
			newx = actualx + newx;
			newy = actualy + newy;
			newwidth = actualwidth + newwidth;
			newheight = actualheight + newheight;
		}

		int counter = 0;
		int deltax = newx - actualx;
		int deltay = newy - actualy;
		int deltasizex = newwidth - actualwidth;
		int deltasizey = newheight - actualheight;

		bool growx;
		bool growy;
		bool growsizex;
		bool growsizey;
		
		if (deltax < 0)
			growx = false;
		else
			growx = true;
		if (deltay < 0)
			growy = false;
		else
			growy = true;
		
		if (deltasizex < 0)
			growsizex = false;
		else
			growsizex = true;
		if (deltasizey < 0)
			growsizey = false;
		else
			growsizey = true;

		div_t div_result;
		div_result = div( deltax, steps );
		deltax = div_result.quot;
		div_result = div( deltay, steps );
		deltay = div_result.quot;

		div_result = div( deltasizex, steps );
		deltasizex = div_result.quot;
		div_result = div( deltasizey, steps );
		deltasizey = div_result.quot;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		if (deltasizex == 0 && growsizex)
			deltasizex = 1;
		else if (deltasizex == 0 && !growsizex)
			deltasizex = -1;
		if (deltasizey == 0 && growsizey)
			deltasizey = 1;
		else if (deltasizey == 0 && !growsizey)
			deltasizey = -1;

		int dynx = actualx;
		int dyny = actualy;
		int dynwidth = actualwidth;
		int dynheight = actualheight;

		if(hWnd)
		{
			int stopper = 0;
			while (counter <= steps)
			{
				stopper = 0; // Stop if x, y, width and height are finished, stopper = 4
				dynx = dynx+deltax;
				if (growx && dynx > newx)
				{
					dynx = newx;
					stopper++;
				}
				else if (!growx && dynx < newx)
				{
					dynx = newx;
					stopper++;
				}

				dyny = dyny+deltay;
				if (growy && dyny > newy)
				{
					dyny = newy;
					stopper++;
				}
				else if (!growy && dyny < newy)
				{
					dyny = newy;
					stopper++;
				}

				dynwidth = dynwidth+deltasizex;
				if (growsizex && dynwidth > newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}
				else if (!growsizex && dynwidth < newwidth)
				{
					dynwidth = newwidth;
					stopper++;
				}

				dynheight = dynheight+deltasizey;
				if (growsizey && dynheight > newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				else if (!growsizey && dynheight < newheight)
				{
					dynheight = newheight;
					stopper++;
				}
				
				SetWindowPos(hWnd, 0, dynx, dyny, dynwidth, dynheight,
					SWP_NOACTIVATE | SWP_NOZORDER);
				
				if (box != NULL)
					SendMessage(box, WM_PAINT, 0, 0);

				CreateLayout();

				SendMessage(hWnd, WM_PAINT, 0, 0);

				if (stopper == 4)
					counter = steps;

				counter++;

				Sleep(time);
			}

			actualx = newx;
			actualy = newy;
			actualwidth = newwidth;
			actualheight = newheight;

			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
				SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called befire!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}
	else
	{
		if(hWnd)
		{
			if (repositionBy)
			{
				newx = actualx + newx;
				newy = actualy + newy;
				newwidth = actualwidth + newwidth;
				newheight = actualheight + newheight;
			}

			actualx = newx;
			actualy = newy;
			actualheight = newheight;
			actualwidth = newwidth;

			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
				SWP_NOACTIVATE | SWP_NOZORDER);

			//Force Repaint to get correct values for Resize Command (SetEvar must be called befire!)
			SendMessage(hWnd, WM_PAINT, 0, 0);
		}
	}

	//updateRegion to track Magic Pink Changes is inside CreateLayout()
	CreateLayout();

	if(!taskbarMove.empty())
		LSExecute(hWnd, taskbarMove.c_str(), SW_SHOWNORMAL);

	if(!taskbarResize.empty())
		LSExecute(hWnd, taskbarResize.c_str(), SW_SHOWNORMAL);

	//Set new Rect for ToolTip Tracker
	if (showToolTip && ToolTips)
		updateHint(hWnd, "", true);
}

void Taskbar::updateRegion()
{
	if (transparencyNeeded)
	{
		HDC tmpDC = CreateCompatibleDC(0);
		HBITMAP tmpBmp = CreateBitmap(actualwidth, actualheight, 1, 32, NULL);
		HGDIOBJ tmpObj = SelectObject(tmpDC, tmpBmp);

		onPaint(tmpDC);
				
		SelectObject(tmpDC, tmpObj);

		HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
		HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(windowRgn, region, NULL, RGN_COPY);

		SetWindowRgn(hWnd, windowRgn, true);	

		DeleteObject(windowRgn);
		DeleteObject(region);
		DeleteObject(tmpBmp);
		DeleteObject(tmpObj);
		DeleteDC(tmpDC);
	}
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

void Taskbar::onLButtonDown(int x, int y)
{
	TaskbarButton *button = ButtonAtPoint( x, y );

	if ( button )
	{
		button->onButtonClick( x, y, lButtonAction );
		movecapture = button;
	}
	else if ( !button && !taskbarLButtonDown.empty() )
	{
		LSExecute(hWnd, taskbarLButtonDown.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onLButtonUp(int x, int y)
{	
	TaskbarButton *button = ButtonAtPoint( x, y );

	if ( button && movecapture != button )
	{	
		vector<TaskbarButton *> tmptaskbuttons;
		
		for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
		{
			if ( *tbit == button )
			{
				if ( button->position < movecapture->position )
				{
					tmptaskbuttons.insert( tmptaskbuttons.end(), (movecapture) );
					tmptaskbuttons.insert( tmptaskbuttons.end(), (button) );
				}
				else if ( button->position > movecapture->position )
				{
					tmptaskbuttons.insert( tmptaskbuttons.end(), (button) );
					tmptaskbuttons.insert( tmptaskbuttons.end(), (movecapture) );
				}
			}
			else if ( *tbit != movecapture )
				tmptaskbuttons.insert( tmptaskbuttons.end(), (*tbit) );
		}
	
		taskbuttons = tmptaskbuttons;
		tmptaskbuttons.clear();
		movecapture = NULL;
		CreateLayout();
		repaint(true);
	}
	else if ( !button && !taskbarLButtonUp.empty() )
	{
		LSExecute(hWnd, taskbarLButtonUp.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onMButtonDown(int x, int y)
{
	TaskbarButton *button = ButtonAtPoint( x, y );

	if ( button )
	{
		button->onButtonClick( x, y, mButtonAction );
	}
	else if ( !button && !taskbarMButtonDown.empty() )
	{
		LSExecute(hWnd, taskbarMButtonDown.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onMButtonUp(int x, int y)
{
	TaskbarButton *button = ButtonAtPoint( x, y );

	if ( !button && !taskbarMButtonUp.empty() )
	{
		LSExecute(hWnd, taskbarMButtonUp.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onRButtonDown(int x, int y)
{
	TaskbarButton *button = ButtonAtPoint( x, y );

	if ( button )
	{
		button->onButtonClick( x, y, rButtonAction );
	}
	else if ( !button && !taskbarRButtonDown.empty() )
	{
		LSExecute(hWnd, taskbarRButtonDown.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onRButtonUp(int x, int y)
{
	TaskbarButton *button = ButtonAtPoint( x, y );

	if ( !button && !taskbarRButtonUp.empty() )
	{
		LSExecute(hWnd, taskbarRButtonUp.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onWheelDown(int x, int y)
{
	if(!taskbarMWheelDown.empty())
		LSExecute(hWnd, taskbarMWheelDown.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Taskbar::onWheelUp(int x, int y)
{
	if(!taskbarMWheelUp.empty())
		LSExecute(hWnd, taskbarMWheelUp.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Taskbar::onMouseEnter()
{
	if(!taskbarEnter.empty())
		LSExecute(hWnd, taskbarEnter.c_str(), SW_SHOWNORMAL);

	if (focusOnEnter == 1)
	{
		focusOnEnter = 2;
		mouse_event(MOUSEEVENTF_MIDDLEDOWN, NULL, NULL, NULL, NULL);
		mouse_event(MOUSEEVENTF_MIDDLEUP, NULL, NULL, NULL, NULL);
	}
}

void Taskbar::onMouseLeave()
{
	if(!taskbarLeave.empty())
		LSExecute(hWnd, taskbarLeave.c_str(), SW_SHOWNORMAL);
}

void Taskbar::onMouseMove(int x, int y)
{
	if(!mouseInside)
	{
		mouseInside = true;
		onMouseEnter();
		SetTimer(hWnd, TIMER_MOUSETRACK, 250, 0);
	}
}

void Taskbar::repaint(bool invalidateCache)
{
	if(hWnd)
	{
		if(invalidateCache)
		{
			if(backgroundDC && backgroundBitmap)
			{
				backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);
				DeleteObject(backgroundBitmap);
				backgroundBitmap = 0;
			}

			if(bufferDC && bufferBitmap)
			{
				bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
				DeleteObject(bufferBitmap);
				bufferBitmap = 0;
			}
		}

		InvalidateRect(hWnd, NULL, false);
	}
}

void Taskbar::onPaint(HDC hDC)
{
	RECT mainRect;
	GetWindowRect(hWnd, &mainRect);

	int width = mainRect.right - mainRect.left;
	int height = mainRect.bottom - mainRect.top;

	RECT completeFillRect;
	completeFillRect.left = 0;
	completeFillRect.top = 0;
	completeFillRect.right = width;
	completeFillRect.bottom = height;

	// keep a cached rendition of the background
	if(!backgroundBitmap)
	{
		if(!backgroundDC)
			backgroundDC = CreateCompatibleDC(hDC);

		backgroundBitmap = CreateCompatibleBitmap(hDC, width, height);
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);

		//True Transparency
		if (transparencyMode)
		{
			::SetBkColor(backgroundDC, RGB(255, 0, 255));
			::ExtTextOut(backgroundDC, 0, 0, ETO_OPAQUE, &completeFillRect, NULL, 0, NULL);
		}
		//PrePaint Background (Fake Transparency)
		else if (!transparencyMode)
		{
			if(box)
			{
				//  =======================================================================
				//  Copy boxbg to backgroundDC
				//  =======================================================================
				BitBlt(backgroundDC, 0, 0, width, height, hDC, 0, 0, SRCCOPY);
			}
			else
			{
				//  =======================================================================
				//  Paint desktop on display DC and then into background buffer
				//  =======================================================================
				PaintDesktopEx(backgroundDC, 0, 0, width, height, mainRect.left, mainRect.top, FALSE);
			}
		}
		
		//Normal Background Painting
		taskbartexture->apply(backgroundDC, 0, 0, width, height, transparencyMode);
	}

	// double buffer for flicker-free paint
	if(!bufferBitmap)
	{
		if(!bufferDC)
			bufferDC = CreateCompatibleDC(hDC);

		bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
	}

	// blt background into double buffer
	BitBlt(bufferDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

	bool displayButton = true;
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_NORMAL
				&& displayButtonType != 0 && displayButtonType != 3 && displayButtonType != 4)
			displayButton = false;
		else if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_SELECTED
				&& displayButtonType != 1 && displayButtonType != 3 && displayButtonType != 5)
			displayButton = false;
		else if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_MINIMIZED
				&& displayButtonType != 2 && displayButtonType != 4 && displayButtonType != 5)
			displayButton = false;

		if (displayButton)
			(*tbit)->onPaint( bufferDC );

		displayButton = true;
	}
	
	// blt the double buffer to the display
	BitBlt(hDC, 0, 0, width, height, bufferDC, 0, 0, SRCCOPY);
}

void Taskbar::onTimer(int timerID)
{
	switch(timerID)
	{	
		case TIMER_UPDATE:

			if(!inCriticalProcess && visible)
			{
				bool needrepaint = false;
				for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
				{
					/*if ((*tbit)->getCurrentState() == STATE_MINIMIZED && minimizeToTray && !(*tbit)->trayed)
					{
						(*tbit)->trayed = true;
						TraySend((*tbit)->hButtonTask);
						needrepaint = true;
					}*/

					if ( (*tbit)->GetFlashing() || (*tbit)->onUpdate(false) )
						needrepaint = true;
				}

				if ( needrepaint )
					repaint();
			}

			break;

		case TIMER_MOUSETRACK:

			POINT pt;
			GetCursorPos(&pt);
			
			RECT rc;
			GetWindowRect(hWnd, &rc);

			if(!PtInRect(&rc, pt))
			{
				KillTimer(hWnd, TIMER_MOUSETRACK);
				mouseInside = false;
				onMouseLeave();	
				capture = NULL;
				updateHint(hWnd, "", false);
				PrefixedLSSetVariable(name, "HoverTask", "");
				if (useHoverState)
					repaint();
				break;
			}

			if (showToolTip || useHoverState)
			{
				ScreenToClient(hWnd, &pt);
				TaskbarButton *button = ButtonAtPoint( pt.x, pt.y );

				if ( button && button != capture )
				{
					updateHint(hWnd, (char *) button->tooltip.c_str(), false);
					PrefixedLSSetVariable(name, "HoverTask", button->caption);
					if (useHoverState)
					{
						button->useHover = true;
						repaint();	
					}
				}
				else if (button == NULL)
				{
					updateHint(hWnd, "", false);
					PrefixedLSSetVariable(name, "HoverTask", "");
					if (useHoverState)
						repaint();
				}
				capture = button;
			}

		break;
	}
}

void Taskbar::onWindowPosChanged(WINDOWPOS *windowPos)
{
	actualx = windowPos->x;
	actualy = windowPos->y;

	repaint(true);
	
	POINT pos = {windowPos->x, windowPos->y};
	POINT size = {windowPos->cx, windowPos->cy};

	POINT boxConverted;
	boxConverted.x = windowPos->x;
	boxConverted.y = windowPos->y;
	if (box != NULL)
		ScreenToClient(box, &boxConverted);

	PrefixedLSSetVariable(name, "CurrentX", IntToString( boxConverted.x ));
	PrefixedLSSetVariable(name, "CurrentY", IntToString( boxConverted.y ));
}

void Taskbar::onSize(int width, int height)
{
	actualheight = height;
	actualwidth = width;

	repaint(true);

	PrefixedLSSetVariable(name, "CurrentWidth", IntToString( width ));
	PrefixedLSSetVariable(name, "CurrentHeight", IntToString( height ));
}

BOOL Taskbar::onWindowMessage( UINT nMessage, WPARAM wParam, LPARAM lParam, LRESULT &lResult )
{
	lResult = 0;

	/*if ( (0x8000 & GetKeyState(VK_CONTROL)) && (0x8000 & GetKeyState(VK_SHIFT)) )
		pressedModifierKey = 3;
	else if ( (0x8000 & GetKeyState(VK_SHIFT)) )
		pressedModifierKey = 2;
	else if ((0x8000 & GetKeyState(VK_CONTROL)) )
		pressedModifierKey = 1;
	else
		pressedModifierKey = 0;*/

	switch ( nMessage )
	{
		//Trayed
		/*case WM_ICON_NOTIFY:
		{ 
			onTray( wParam, lParam );
		}*/

		case WM_CLOSE:
		{
			lResult = 0;
			return TRUE;
		}

		case LM_LSSELECT:
		{
			onWindowActivated( (HWND) wParam, FALSE );
			if (displayButtonType != -1)
				CreateLayout();
			else
			{
				if (transparencyMode)
					updateRegion();
			}
			return TRUE;
		}

		case LM_REDRAW:
		{
			onRedraw( (HWND) wParam, (BOOL) lParam ); 

			return TRUE;
		}

		case LM_WINDOWACTIVATED:
		{
			onWindowActivated( (HWND) wParam, (BOOL) lParam );
			if (displayButtonType != -1)
				CreateLayout();
			else
			{
				if (transparencyMode)
					updateRegion();
			}
			return TRUE;
		}

		case LM_WINDOWCREATED:
		{
			onWindowCreated( (HWND) wParam );
			CreateLayout();
			return TRUE;
		}

		case LM_WINDOWDESTROYED:
		{
			onWindowDestroyed( (HWND) wParam );
			CreateLayout();
			return TRUE;
		}

		case WM_LBUTTONDOWN:
		{
			onLButtonDown( LOWORD(lParam), HIWORD(lParam));
			//pressedModifierKey = 0;
			return TRUE;
		}

		case WM_LBUTTONUP:
		{
			onLButtonUp( LOWORD(lParam), HIWORD(lParam));
			//pressedModifierKey = 0;
			return TRUE;
		}

		case WM_MBUTTONDOWN:
		{
			if (focusOnEnter == 2)
				return true;

			onMButtonDown( LOWORD(lParam), HIWORD(lParam));
			//pressedModifierKey = 0;
			return TRUE;
		}

		case WM_MBUTTONUP:
		{
			if (focusOnEnter == 2)
			{
				focusOnEnter = 1;
				return true;
			}

			onMButtonUp( LOWORD(lParam), HIWORD(lParam));
			//pressedModifierKey = 0;
			return TRUE;
		}

		case WM_RBUTTONDOWN:
		{
			onRButtonDown( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			//pressedModifierKey = 0;
			return TRUE;
		}

		case WM_RBUTTONUP:
		{
			onRButtonUp( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			//pressedModifierKey = 0;
			return TRUE;
		}

		case WM_MOUSEWHEEL:
		{
			if ((short)(HIWORD(wParam)) < 0)
			{
				onWheelDown((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
				return true;
			}
			else
			{
				onWheelUp((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
				return true;
			}
		}

		case WM_MOUSEMOVE:
		{
			onMouseMove( LOWORD( lParam ), HIWORD( lParam ) );
			return TRUE;
		}
	
		case WM_PAINT:
		{
			if (!inCriticalProcess && visible)
			{
				PAINTSTRUCT ps;

				HDC hDC;

				hDC = BeginPaint( hWnd, &ps );

				onPaint( hDC );

				EndPaint( hWnd, &ps );
			}

			return TRUE;
		}

		case WM_WINDOWPOSCHANGED:
		{
			onWindowPosChanged((WINDOWPOS *) lParam);
			return false;
		}

		case WM_SIZE:
		{
			onSize( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			return TRUE;
		}

		case WM_TIMER:
		{
			onTimer((int) wParam);
			return true;
		}

		case LM_UPDATEBG:
		{
			PaintDesktopEx(0, 0, 0, 0, 0, 0, 0, TRUE);

			repaint(true);

			return 0;
		}

		case WM_DISPLAYCHANGE:
		{
			return TRUE;
		}

		case WM_SETTINGCHANGE:
		{
			PostMessage(hWnd, LM_UPDATEBG, 0, 0);
			return 0;
		}

		case MOUSEISDRAGGING:
		{
			POINT p;
			p.x = GET_X_LPARAM( lParam );
			p.y = GET_Y_LPARAM( lParam );

			ScreenToClient( hWnd, &p );

			TaskbarButton* button = ButtonAtPoint( p.x, p.y );
			if( button )
			{
				if (button->currentState != STATE_SELECTED)
					SwitchToThisWindow( button->hButtonTask, SW_RESTORE );
			}
		}
	}

	return FALSE;
}

TaskbarButton *Taskbar::ButtonAtPoint( int x, int y )
{
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if ( (*tbit)->HasPoint(x, y) )
		{
			return (*tbit);
		}
	}

	return NULL;
}

TaskbarButton *Taskbar::SearchForButtonByHandle( HWND search_hWnd )
{
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if ( search_hWnd == (*tbit)->hButtonTask  )
			return (*tbit);
	}

	return NULL;
}

TaskbarButton *Taskbar::SearchForButtonByClass( HWND search_hWnd )
{
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if ( stricmp( (GetTaskClass( search_hWnd )).c_str(), (*tbit)->classname.c_str()) == NULL )
			return (*tbit);
	}

	return NULL;
}

BOOL WINAPI Taskbar::EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	Taskbar *taskbar = (Taskbar *) lParam;
	if (taskbar && IsAppWindow(hWnd))
	{
		//Grouping
		TaskbarButton *searchbutton = taskbar->SearchForButtonByClass( hWnd );
		if (searchbutton != NULL && taskbar->useGrouping)
		{
			searchbutton->AddToGroup( hWnd );
		}
		else if (taskbar->SearchForButtonByHandle(hWnd) == NULL && !IsOnTaskbarList(hWnd, RemoveList) && IsOnTaskbarList(hWnd, SpecificList, taskbar->getName()))
		{
			TaskbarButton *button = new TaskbarButton( taskbar, hWnd );
			taskbar->taskbuttons.push_back(button);
		
			++nButtonsAddedAtStartup;
		}
	}
	return TRUE;
}

LRESULT WINAPI Taskbar::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Taskbar *taskbar = (Taskbar *) GetWindowLong(hWnd, 0);

	if (message == WM_NCCREATE)
	{
		LPCREATESTRUCT pcs = (LPCREATESTRUCT) lParam;
		taskbar = (Taskbar *) pcs->lpCreateParams;

		taskbar->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) taskbar);
	}

	if (taskbar)
	{
		LRESULT lResult;

		if (taskbar->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

void Taskbar::CreateLayout()
{
	inCriticalProcess = true;
	int buttonCount = 0;

	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if ((*tbit)->getCurrentState() == STATE_NORMAL /*&& !(*tbit)->trayed*/
				&& (displayButtonType == -1 || displayButtonType == 0 || displayButtonType == 3 || displayButtonType == 4))
		{
			buttonCount++;
			(*tbit)->position = buttonCount;
		}
		else if ((*tbit)->getCurrentState() == STATE_SELECTED /*&& !(*tbit)->trayed*/
				&& (displayButtonType == -1 || displayButtonType == 1 || displayButtonType == 3 || displayButtonType == 5))
		{
			buttonCount++;
			(*tbit)->position = buttonCount;
		}
		else if ((*tbit)->getCurrentState() == STATE_MINIMIZED /*&& !(*tbit)->trayed*/
				&& (displayButtonType == -1 || displayButtonType == 2 || displayButtonType == 4 || displayButtonType == 5))
		{
			buttonCount++;
			(*tbit)->position = buttonCount;
		}
	}

	PrefixedLSSetVariable(name, "CurrentButtonCount", IntToString( buttonCount ));

	if (buttonCount <= 0)
	{
		InvalidateRect( hWnd, NULL, FALSE );
		inCriticalProcess = false;

		if (transparencyMode)
			updateRegion();

		return;
	}

	if (direction <= 2)
	{
	//Start Horizontal Version
	//-----------------------------------------------------------------------
	int maxavailableWidth = actualwidth - leftBorder - rightBorder;
	int availableWidth = max( maxavailableWidth, 0 );  // cannot be less than zero

	if (autoNewLine && (availableWidth - ( buttonCount/(lines_rows+autolines_rows) ) * xSpacing)/( (buttonCount/(lines_rows+autolines_rows)) + (buttonCount%(lines_rows+autolines_rows) != 0 ? 1 : 0) ) < minTaskWidth)
	{	
		autolines_rows++;
		if (!taskbarLineAdd.empty() && lines_rows+autolines_rows <= maxlines_rows)
			LSExecute(hWnd, taskbarLineAdd.c_str(), SW_SHOWNORMAL);

	}
	else if (autoNewLine && (availableWidth - ( buttonCount/(lines_rows+ (autolines_rows >= 1 ? autolines_rows-1 : 0) ) ) * xSpacing)/( (buttonCount/(lines_rows+ (autolines_rows >= 1 ? autolines_rows-1 : 0) )) + (buttonCount%(lines_rows+ (autolines_rows >= 1 ? autolines_rows-1 : 0) ) != 0 ? 1 : 0) ) >= minTaskWidth)
	{
		if (autolines_rows >= 1)
		{
			autolines_rows--;
			if (!taskbarLineRemove.empty())
				LSExecute(hWnd, taskbarLineRemove.c_str(), SW_SHOWNORMAL);
		}
	}

	int realLines = min(lines_rows+autolines_rows, maxlines_rows);

	PrefixedLSSetVariable(name, "CurrentLines", IntToString( realLines ));

	if ( buttonCount > 1 )
	{
		availableWidth = availableWidth - ( buttonCount/realLines ) * xSpacing;
		availableWidth = max( availableWidth, 0 );
	}

	int widthPerButton = min(availableWidth/( ( buttonCount/realLines ) + (buttonCount%realLines != 0 ? 1 : 0) ), maxTaskWidth);
	int buttonHeight = min( (actualheight - topBorder - bottomBorder - (realLines - 1) * ySpacing)/realLines, maxTaskHeight);

	int x = 0;
	int y = 0;

	int buttoncounter = 1;
	int hcounter = 1;
	int vcounter = 1;
	bool displayButton = true;
	// set placement for each button
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_NORMAL
				&& displayButtonType != 0 && displayButtonType != 3 && displayButtonType != 4)
			displayButton = false;
		else if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_SELECTED
				&& displayButtonType != 1 && displayButtonType != 3 && displayButtonType != 5)
			displayButton = false;
		else if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_MINIMIZED
				&& displayButtonType != 2 && displayButtonType != 4 && displayButtonType != 5)
			displayButton = false;

		//Trayed
		//if ( (*tbit)->trayed )
		//	displayButton = false;

		if (displayButton)
		{
			if (direction == 2)
			{
				x = maxavailableWidth + leftBorder - (hcounter) * (widthPerButton + xSpacing);
				y = topBorder + (vcounter-1) * (buttonHeight + ySpacing);
			}
			else
			{
				x = leftBorder + (hcounter-1) * (widthPerButton + xSpacing);
				y = topBorder + (vcounter-1) * (buttonHeight + ySpacing);
				
			}

			(*tbit)->setButtonLayout( x, y, widthPerButton, buttonHeight );

			if (buttoncounter % realLines == 0)
			{
				hcounter++;
				vcounter = 0;
			}
			vcounter++;
			buttoncounter++;
		}

		displayButton = true;
	}
	//-----------------------------------------------------------------------
	//End Horizontal Version
	}
	else if (direction > 2)
	{
	//Start Vertical Version
	//-----------------------------------------------------------------------
	int maxavailableHeight = actualheight - topBorder - bottomBorder;
	int availableHeight = max( maxavailableHeight, 0 );  // cannot be less than zero

	if (autoNewLine && (availableHeight - ( buttonCount/(lines_rows+autolines_rows) ) * ySpacing)/( (buttonCount/(lines_rows+autolines_rows)) + (buttonCount%(lines_rows+autolines_rows) != 0 ? 1 : 0) ) < minTaskHeight)
	{	
		autolines_rows++;
		if (!taskbarLineAdd.empty() && lines_rows+autolines_rows <= maxlines_rows)
			LSExecute(hWnd, taskbarLineAdd.c_str(), SW_SHOWNORMAL);
	}
	else if (autoNewLine && (availableHeight - ( buttonCount/(lines_rows+ (autolines_rows >= 1 ? autolines_rows-1 : 0) ) ) * ySpacing)/( (buttonCount/(lines_rows+ (autolines_rows >= 1 ? autolines_rows-1 : 0) )) + (buttonCount%(lines_rows+ (autolines_rows >= 1 ? autolines_rows-1 : 0) ) != 0 ? 1 : 0) ) >= minTaskHeight)
	{
		if (autolines_rows >= 1)
		{
			autolines_rows--;
			if (!taskbarLineRemove.empty())
				LSExecute(hWnd, taskbarLineRemove.c_str(), SW_SHOWNORMAL);
		}
	}

	int realLines = min(lines_rows+autolines_rows, maxlines_rows);

	PrefixedLSSetVariable(name, "CurrentLines", IntToString( realLines ));

	if ( buttonCount > 1 )
	{
		availableHeight = availableHeight - (buttonCount/realLines) * ySpacing;
		availableHeight = max( availableHeight, 0 );
	}

	int heightPerButton = min(availableHeight/( (buttonCount/realLines) + (buttonCount%realLines != 0 ? 1 : 0) ), maxTaskHeight);
	int buttonWidth = min( (actualwidth - leftBorder - rightBorder - (realLines - 1) * xSpacing)/realLines, maxTaskWidth);

	int x = 0;
	int y = 0;

	int buttoncounter = 1;
	int hcounter = 1;
	int vcounter = 1;
	bool displayButton = true;
	// set placement for each button
	for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
	{
		if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_NORMAL
				&& displayButtonType != 0 && displayButtonType != 3 && displayButtonType != 4)
			displayButton = false;
		else if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_SELECTED
				&& displayButtonType != 1 && displayButtonType != 3 && displayButtonType != 5)
			displayButton = false;
		else if (displayButtonType != -1 && (*tbit)->getCurrentState() == STATE_MINIMIZED
				&& displayButtonType != 2 && displayButtonType != 4 && displayButtonType != 5)
			displayButton = false;

		//Trayed
		//if ( (*tbit)->trayed )
		//	displayButton = false;

		if (displayButton)
		{
			if (direction == 4)
			{
				x = leftBorder + (hcounter-1) * (buttonWidth + xSpacing);
				y = maxavailableHeight + topBorder - vcounter * (heightPerButton + ySpacing);
			}
			else
			{
				x = leftBorder + (hcounter-1) * (buttonWidth + xSpacing);
				y = topBorder + (vcounter-1) * (heightPerButton + ySpacing);
			}

			(*tbit)->setButtonLayout( x, y, buttonWidth, heightPerButton );

			if (buttoncounter % realLines == 0)
			{
				vcounter++;
				hcounter = 0;
			}
			hcounter++;
			buttoncounter++;
		}

		displayButton = true;
	}
	//-----------------------------------------------------------------------
	//End Horizontal Version
	}
	inCriticalProcess = false;
	
	repaint();

	if (transparencyMode)
		updateRegion();
}

void Taskbar::onRedraw( HWND hApplication, bool fFlash )
{
	TaskbarButton *button = SearchForButtonByHandle(hApplication);

	if (button)
	{
		if(GetActiveWindow() != button->hButtonTask)
		{
			button->SetFlashing(fFlash);
			if (fFlash)
			{
				KillTimer(hWnd, TIMER_UPDATE);
				SetTimer(hWnd, TIMER_UPDATE, flashRefresh, NULL);
			}
		}
	}
}

void Taskbar::onWindowActivated( HWND hApplication, BOOL fFullScreen )
{
	TaskbarButton *searchbutton = SearchForButtonByClass( hApplication );
	if (searchbutton && useGrouping)
	{
		searchbutton->hButtonTask = hApplication;
		if (searchbutton->GetFlashing())
		{
			KillTimer(hWnd, TIMER_UPDATE);
			SetTimer(hWnd, TIMER_UPDATE, timerRefresh, NULL);
		}
		searchbutton->SetFlashing(FALSE);
		searchbutton->currentState = searchbutton->getCurrentState();
	}
	else
	{
		TaskbarButton * searchbutton = SearchForButtonByHandle(hApplication);
		if (searchbutton)
		{
			if (searchbutton->GetFlashing())
			{
				KillTimer(hWnd, TIMER_UPDATE);
				SetTimer(hWnd, TIMER_UPDATE, timerRefresh, NULL);
			}
			searchbutton->SetFlashing(FALSE);
			searchbutton->currentState = searchbutton->getCurrentState();	
		}
	}
	
	repaint();
}

void Taskbar::onWindowCreated( HWND hApplication )
{
	//Grouping
	TaskbarButton *searchbutton = SearchForButtonByClass( hApplication );
	if (searchbutton != NULL && useGrouping)
	{
		searchbutton->AddToGroup( hApplication );
	}
	else if (SearchForButtonByHandle(hApplication) == NULL && !IsOnTaskbarList(hApplication, RemoveList) && IsOnTaskbarList(hApplication, SpecificList, name))
	{
		TaskbarButton *button = new TaskbarButton( this, hApplication );
		taskbuttons.insert(taskbuttons.end(), button);

		if(!taskbarAdd.empty())
			LSExecute(hWnd, taskbarAdd.c_str(), SW_SHOWNORMAL);
	}
}

void Taskbar::onWindowDestroyed( HWND hApplication )
{
	//Grouping
	TaskbarButton *searchbutton = SearchForButtonByClass( hApplication );
	if (searchbutton != NULL)
	{
		if ( searchbutton->hwndgrouplist.size() > 1 && useGrouping)
		{
			searchbutton->RemoveFromGroup( hApplication );
		}
		else
		{
			for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
			{
				if ( (*tbit)->hButtonTask == hApplication )
				{
					delete (*tbit);
					taskbuttons.erase( tbit );
					
					if(!taskbarRemove.empty())
						LSExecute(hWnd, taskbarRemove.c_str(), SW_SHOWNORMAL);
					break;
				}
			}
		}
	}
	else
	{
		for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit )
		{	
			if ( (*tbit)->hButtonTask == hApplication )
			{
				delete (*tbit);
				taskbuttons.erase( tbit );

				if(!taskbarRemove.empty())
					LSExecute(hWnd, taskbarRemove.c_str(), SW_SHOWNORMAL);
				break;
			}
		}
	}
}

void Taskbar::updateHint(HWND hWnd, LPSTR caption, bool updatesize, bool remove)
{
	if (ToolTips == 0)
		return;	
	
	if (!remove)
	{
		TOOLINFO ti;
		RECT clientRect;

		GetClientRect(hWnd, &clientRect);
		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = TTF_SUBCLASS;
		ti.hwnd = hWnd;
		ti.uId = 0;
		ti.rect = clientRect;
		ti.hinst = hInstance;
		ti.lpszText = caption;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_UPDATETIPTEXT, 0, LPARAM(&ti));

		if (updatesize)
			SendMessage(ToolTips, TTM_NEWTOOLRECT, 0, LPARAM(&ti));
	}
	else
	{
		TOOLINFO ti;
		RECT emptyRect = {0, 0, 0, 0};

		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = 0;
		ti.hwnd = hWnd;
		ti.uId = 0;
		ti.rect = emptyRect;
		ti.hinst = hInstance;
		ti.lpszText = NULL;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_DELTOOL, 0, LPARAM(&ti));
	}
}

/*void Taskbar::TraySend(HWND window)
{
	BOOL IconNotified = false;

	HICON tmpIcon = GetTaskIcon(window);

	NOTIFYICONDATA Icon;
	ZeroMemory(&Icon, sizeof(Icon));
		
	Icon.cbSize = sizeof(NOTIFYICONDATA); //NOTIFYICONDATA_V1_SIZE // sizeof(NOTIFYICONDATA)
	Icon.hWnd = hWnd; // use this HWND to accept messages
	Icon.uID = (int) window;
	Icon.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP; //  
	Icon.uCallbackMessage = WM_ICON_NOTIFY; // WM_ICON_NOTIFY
	Icon.hIcon = tmpIcon;

	GetWindowText( window, Icon.szTip, sizeof(Icon.szTip) );

	IconNotified = Shell_NotifyIcon(NIM_ADD, &Icon);
	if (IconNotified)
	{
		for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
		{
			if ( (*tbit)->hButtonTask == window )
			{
				(*tbit)->trayed = true;
				break;
			}
		}

		ShowWindow( window, SW_HIDE );
		hWndTrayedList.push_back(window);
	}

	DestroyIcon(tmpIcon);
}

void Taskbar::TrayRestore(HWND window)
{
	BOOL IconNotified = false;

	HICON tmpIcon = GetTaskIcon(window);

	NOTIFYICONDATA Icon;
	ZeroMemory(&Icon, sizeof(Icon));
		
	Icon.cbSize = sizeof(NOTIFYICONDATA); //NOTIFYICONDATA_V1_SIZE // sizeof(NOTIFYICONDATA)
	Icon.hWnd = hWnd; // use this window to accept messages
	Icon.uID = (int) window;
	Icon.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP; //  
	Icon.uCallbackMessage = WM_ICON_NOTIFY; // WM_ICON_NOTIFY
	Icon.hIcon = tmpIcon;

	GetWindowText( window, Icon.szTip, sizeof(Icon.szTip) );

	IconNotified = Shell_NotifyIcon(NIM_DELETE, &Icon);
	if (IconNotified)
	{
		for (TaskbarButtonIterator tbit = taskbuttons.begin(); tbit != taskbuttons.end(); ++tbit)
		{
			if ( (*tbit)->hButtonTask == window )
			{
				(*tbit)->currentState = STATE_SELECTED;
				(*tbit)->trayed = false;
				break;
			}
		}

		ShowWindow(window, SW_SHOW );
		SelectWindow( window );
		hWndTrayedList.remove(window);
	}

	DestroyIcon(tmpIcon);
}

void Taskbar::TrayRestoreAll()
{
	for (list<HWND>::iterator i = hWndTrayedList.begin(); i != hWndTrayedList.end(); i++)
	{
		TrayRestore( (*i) );
	}

	hWndTrayedList.clear();
}

void Taskbar::onTray( WPARAM wParam, LPARAM lParam )
{
	switch(lParam)
	{
		case WM_LBUTTONUP:
		
			TrayRestore( (HWND)wParam );
			break;

		case WM_RBUTTONUP:

			TrayRestore( (HWND)wParam );
			break;

		default:

			break;
	}
}*/
