#include "stdafx.h"
#include "Utilities.h"

extern vector<string> RemoveList;
extern vector<string> CustomIconList;
extern vector<string> SpecificList;

HDC hdcDesktop = 0;
HBITMAP hbmDesktop = 0;

string GetTaskClass( HWND hTask )
{
	char windowClass[MAX_PATH_LENGTH];
	strcpy(windowClass, "");

	if (hTask)
		GetClassName( hTask, windowClass, MAX_PATH_LENGTH );

	return (string)windowClass;
}

void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

bool IsSubString(string big, string small)
{
	string::iterator itBig = big.begin();
	string::iterator itSmall = small.begin();

	while(itBig != big.end() && itSmall != small.end())
	{
		if(*itBig == *itSmall)
			++itSmall;
        ++itBig;
	}
	return(itSmall == small.end());
}

bool IsOnTaskbarList(HWND hWnd, vector<string> theList, string taskbarname)
{
	char tmpTitle[MAX_PATH_LENGTH], tmpClass[MAX_PATH_LENGTH];

	GetWindowText(hWnd, tmpTitle, MAX_PATH_LENGTH);
	GetClassName(hWnd, tmpClass, MAX_PATH_LENGTH);

	if (taskbarname.empty())
	{
		for (vector<string>::iterator it = theList.begin();it != theList.end();++it)
		{
			char search[MAX_PATH_LENGTH];
			char *buffers[] = {search};

			LCTokenize((*it).c_str(), buffers, 1, NULL);

			if (IsSubString(tmpTitle, search) || IsSubString(tmpClass, search))
				return TRUE;
		}
		return FALSE;
	}
	else
	{
		for (vector<string>::iterator it = theList.begin();it != theList.end();++it)
		{
			char name[64], search[MAX_PATH_LENGTH];
			char *buffers[] = {name, search};

			LCTokenize((*it).c_str(), buffers, 2, NULL);

			if ( (IsSubString(tmpTitle, search) || IsSubString(tmpClass, search)) && stricmp(name, taskbarname.c_str()) != NULL )
				return FALSE;
		}
		return TRUE;
	}
}

HICON GetTaskIcon(HWND hTask, bool bBigIcon)
{
	HICON hIcon = NULL;

	if (hTask)
	{
		SendMessageTimeout( hTask, WM_GETICON, bBigIcon ? ICON_BIG : ICON_SMALL, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
		if ( !hIcon )
			hIcon = (HICON) GetClassLong( hTask, bBigIcon ? GCL_HICON : GCL_HICONSM);
		if ( !hIcon )
			SendMessageTimeout( hTask, WM_QUERYDRAGICON, 0, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
		if ( !hIcon )
			SendMessageTimeout( hTask, WM_GETICON, bBigIcon ? ICON_SMALL : ICON_BIG, 0, SMTO_ABORTIFHUNG, 1000, (LPDWORD) &hIcon );
		if ( !hIcon )
			hIcon = (HICON) GetClassLong( hTask, bBigIcon ? GCL_HICONSM : GCL_HICON);
	}
	if(!hIcon)
		hIcon = LoadIcon(NULL, MAKEINTRESOURCE(IDI_APPLICATION));

	return hIcon;
}

HICON GetIconOnCustomList(HWND hWnd, vector<string> theList, bool bBigIcon)
{
	char tmpTitle[MAX_PATH_LENGTH], tmpClass[MAX_PATH_LENGTH];

	GetWindowText(hWnd, tmpTitle, MAX_PATH_LENGTH);
	GetClassName(hWnd, tmpClass, MAX_PATH_LENGTH);

	for (vector<string>::iterator it = theList.begin();it != theList.end();++it)
	{
		char search[64], iconPath[MAX_PATH_LENGTH];
		char *buffers[] = {search, iconPath};

		LCTokenize((*it).c_str(), buffers, 2, NULL);
		
		if (IsSubString(tmpTitle, search) || IsSubString(tmpClass, search))
		{
			SHFILEINFO shFileInfo;
			SHGetFileInfo(iconPath, NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (bBigIcon ? SHGFI_LARGEICON : SHGFI_SMALLICON) );

			return shFileInfo.hIcon;
		}
	}

	return NULL;
}

bool IsOS(const int &os)
{	
	static int osversion = 0;

	static bool bInitialized = false;

	if (!bInitialized)
	{
		OSVERSIONINFO verInfo;
		verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx(&verInfo);
		
		switch(verInfo.dwPlatformId)
		{
			case VER_PLATFORM_WIN32_WINDOWS:

				osversion |= OS_9X;
			
				if(verInfo.dwMinorVersion >= 90)
				{
					osversion |= OS_ME;
				}
				else if(verInfo.dwMinorVersion >= 10)
				{
					osversion |= OS_98;
				}
				else
				{
					osversion |= OS_95;
				}
				break;
			
			case VER_PLATFORM_WIN32_NT:
			
				osversion |= OS_NT;
			
				if(verInfo.dwMajorVersion >= 5)
				{
					osversion |= OS_2KXP;

					if (verInfo.dwMinorVersion >= 1)
					{
						osversion |= OS_XP;
					}
					else
					{
						osversion |= OS_2K;
					}
				}
				else
				{
					osversion |= OS_NT4;
				}
				break;
			default:;
		}
		bInitialized = true;
	}

	return (osversion & os) ? true : false;
}

// retrieve handle of the desktop window
HWND GetLitestepDesktop()
{
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);

	if(hWnd != 0)
		return hWnd;

	return GetDesktopWindow();
}

// is the given window a toplevel application window?
bool IsAppWindow(const HWND &hWnd)
{
	if( IsOnTaskbarList(hWnd, RemoveList) )
		return FALSE;

	if(!IsWindowVisible(hWnd))
		return false;

	if(GetWindow(hWnd, GW_OWNER) != 0)
		return false;

	long exStyle = GetWindowLong(hWnd, GWL_EXSTYLE);

	if(exStyle & WS_EX_APPWINDOW)
		return true;

	if(exStyle & WS_EX_TOOLWINDOW)
		return false;

	if(GetWindowLong(hWnd, GWL_USERDATA) == magicDWord)
		return false;

	return true;
}

// modify window style
void ModifyStyle(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_STYLE, (GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}

void ModifyStyleEx(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_EXSTYLE, (GetWindowLong(hWnd, GWL_EXSTYLE) & ~removeStyle) | addStyle);
}

// trim leading and trailing whitespace
string trim(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

void GetxRCList(char *text, vector<string> &theList)
{
	FILE * f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, text, lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				theList.insert(theList.end(), configline);
			}
		}
		
		LCClose(f);
	}
}

// retrieve a boolean value from the step.rc
bool GetxRCBool(const string &object, const string &subKey, bool defaultVal)
{
	string key = object + subKey;
	return GetRCBoolDef(key.c_str(), defaultVal) ? true : false;
}

// retrieve a string value from step.rc
string GetxRCString(const string &object, const string &subKey, const string &defaultVal)
{
	string key = object + subKey;
	char buffer[MAX_LINE_LENGTH];
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);
	return string(buffer);
}

// retrieve a line from step.rc
string GetxRCLine(const string &object, const string &subKey, const string &defaultVal)
{
	string key = object + subKey;
	char buffer[MAX_LINE_LENGTH];
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, defaultVal.c_str());
	return string(buffer);
}

// retrieve a named value from step.rc
int GetxRCNamedValue(const string &prefix, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	string name = GetxRCString(prefix, baseName, "");
	const NameValuePair *current = nameValuePairs;

	if(name.length() > 0)
	{
		while(current->name)
		{
			if(_stricmp(current->name, name.c_str()) == 0)
				return current->value;

			current++;
		}
	}

	return defaultVal;
}

// retrieve a color value from step.rc
int GetxRCColor(const string &object, const string &subKey, int defaultVal)
{
	string key = object + subKey;
	return GetRCColor(key.c_str(), defaultVal);
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int GetxRCInt(const string &object, const string &subKey, int defaultVal, int minVal, int maxVal)
{
	string key = object + subKey;
	int answer = GetRCInt(key.c_str(), defaultVal);
	return max(min(answer, maxVal), minVal);
}

// parse a coordinate specification
int xParseCoord(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trim(aString);

	if(strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool realnegative = false;
	bool center = false;
	UINT i = 0;
	int value = 0;

	if(strVal[i] == '-')
	{
		negative = true;
		i++;
	}
	else if(strVal[i] == '~')
	{
		realnegative = true;
		i++;
	}
	else if(strVal[i] == '+')
	{
		i++;
	}

	while(i < strVal.length())
	{
		if(strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if(strVal[i] == 'c' || strVal[i] == 'C')
				center = true;

			break;
		}

		i++;
	}

	if(center)
	{
		if (realnegative)
			negative = true;

		if(negative)
			value = (maxVal / 2) - value;
		else
			value = (maxVal / 2) + value;
	}
	else if(realnegative)
	{
		value = -value;
	}
	else if(negative)
	{
		value = maxVal - value;
	}

	return value;
}

// retrieve a coordinate from step.rc, accounting for negative values and centering
int GetxRCCoord(const string &prefix, const string &baseName, int defaultVal, int maxVal)
{
	string strVal = GetxRCString(prefix, baseName, "");
	return xParseCoord(strVal, defaultVal, maxVal);
}

// parse a dimension
int xParseDimen(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trim(aString);

	if(strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	UINT i = 0;
	int value = 0;

	if(strVal[i] == '-')
	{
		negative = true;
		i++;
	}

	while(i < strVal.length())
	{
		if(strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if(strVal[i] == '%')
				percentage = true;

			break;
		}

		i++;
	}

	if(percentage)
		value = (value * maxVal) / 100;

	if(negative)
		value = maxVal - value;

	return value;
}

// retrieve a dimension (absolute value or percentage) from step.rc
int GetxRCDimen(const string &prefix, const string &baseName, int defaultVal, int maxVal)
{
	string strVal = GetxRCString(prefix, baseName, "");
	return xParseDimen(strVal, defaultVal, maxVal);
}

// enhanced version of PaintDesktop
void PaintDesktopEx(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, int xSrc, int ySrc, BOOL updateCache)
{
	if(hdcDesktop == 0 || updateCache)
	{
		if(hdcDesktop == 0 && hdcDest == 0)
			return;

		if(hbmDesktop != 0)
		{
			hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);
			DeleteObject(hbmDesktop);
			hbmDesktop = 0;
		}

		HDC hdcScreen = GetDC(0);

		if(hdcDesktop == 0)
		{
			hdcDesktop = CreateCompatibleDC(hdcScreen);
		}
		
		int cxDesktop = GetSystemMetrics(SM_CXSCREEN);
		int cyDesktop = GetSystemMetrics(SM_CYSCREEN);

		hbmDesktop = CreateCompatibleBitmap(hdcScreen, cxDesktop, cyDesktop);
		hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);

		RECT r;
		SetRect(&r, 0, 0, cxDesktop, cyDesktop);
		FillRect(hdcDesktop, &r, GetSysColorBrush(COLOR_DESKTOP));

		char wallpaper[MAX_PATH_LENGTH] = { 0 };

		RegQueryStringValue(HKEY_CURRENT_USER,
			"Control Panel\\Desktop", "Wallpaper", wallpaper,
			MAX_PATH_LENGTH, "");

		if(wallpaper[0])
		{
			int wallpaperStyle;
			int tileWallpaper;

			char buffer[32];

			RegQueryStringValue(HKEY_CURRENT_USER, "Control Panel\\Desktop", "WallpaperStyle", buffer, 32, "0");
			wallpaperStyle = atoi(buffer);

			RegQueryStringValue(HKEY_CURRENT_USER, "Control Panel\\Desktop", "TileWallpaper", buffer, 32, "0");
			tileWallpaper = atoi(buffer);

			HBITMAP hbmWallpaper = (HBITMAP) LoadImage(0, wallpaper,
				IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

			if(hbmWallpaper)
			{
				BITMAP bm;
				GetObject(hbmWallpaper, sizeof(BITMAP), &bm);

				int cxWallpaper = bm.bmWidth;
				int cyWallpaper = bm.bmHeight;

				HDC hdcWallpaper = CreateCompatibleDC(hdcScreen);
				hbmWallpaper = (HBITMAP) SelectObject(hdcWallpaper, hbmWallpaper);

				if(tileWallpaper)
				{
					TileBlt(hdcDesktop,
						0, 0,
						cxDesktop, cyDesktop,
						hdcWallpaper,
						0, 0,
						cxWallpaper, cyWallpaper,
						SRCCOPY);
				}
				else
				{
					if(wallpaperStyle == 2) // stretch
					{
						SetStretchBltMode(hdcDesktop, STRETCH_DELETESCANS);

						StretchBlt(hdcDesktop,
							0, 0,
							cxDesktop, cyDesktop,
							hdcWallpaper,
							0, 0,
							cxWallpaper, cyWallpaper,
							SRCCOPY);
					}
					else
					{
						BitBlt(hdcDesktop,
						(cxDesktop - cxWallpaper) / 2,
						(cyDesktop - cyWallpaper) / 2,
						cxWallpaper, cyWallpaper,
						hdcWallpaper,
						0, 0,
						SRCCOPY);
					}
				}

				hbmWallpaper = (HBITMAP) SelectObject(hdcWallpaper, hbmWallpaper);
				DeleteDC(hdcWallpaper);

				DeleteObject(hbmWallpaper);
			}
		}

		ReleaseDC(0, hdcScreen);
	}

	if(hdcDest)
	{
		BitBlt(hdcDest, xDest, yDest, cxDest, cyDest, hdcDesktop, xSrc, ySrc, SRCCOPY);
	}
}

// tile an image horizontally and vertically
void TileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		for(int x = 0; x < cxDest; x += cxSrc)
		{
			BitBlt(hdcDest,
				xDest + x,
				yDest + y,
				min(cxDest - x, cxSrc),
				min(cyDest - y, cySrc),
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}
	}
}

// tile an image horizontally
void HorizontalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int x = 0; x < cxDest; x += cxSrc)
	{
		StretchBlt(hdcDest,
			xDest + x,
			yDest,
			min(cxDest - x, cxSrc),
			cyDest,
			hdcSrc,
			xSrc,
			ySrc,
			min(cxDest - x, cxSrc),
			cySrc,
			rasterOp);
	}
}

// tile an image vertically
void VerticalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		StretchBlt(hdcDest,
			xDest,
			yDest + y,
			cxDest,
			min(cyDest - y, cySrc),
			hdcSrc,
			xSrc,
			ySrc,
			cxSrc,
			min(cyDest - y, cySrc),
			rasterOp);
	}
}

// multi-blt!
void MultiBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, int cxLeft, int cyTop, int cxRight, int cyBottom, int mode, DWORD rasterOp)
{
	if(cyTop > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				yDest,
				cxLeft,
				cyTop,
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				yDest,
				cxRight,
				cyTop,
				hdcSrc,
				cxSrc - cxRight,
				ySrc,
				rasterOp);
		}
	}

	if(cxLeft > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(mode == MULTIBLT_STRETCH)
	{
		StretchBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILE)
	{
		TileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEHORIZONTAL)
	{
		HorizontalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEVERTICAL)
	{
		VerticalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}

	if(cxRight > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(cyBottom > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				cyDest - cyBottom,
				cxLeft,
				cyBottom,
				hdcSrc,
				xSrc,
				cySrc - cyBottom,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				cyDest - cyBottom,
				cxRight,
				cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				cySrc - cyBottom,
				rasterOp);
		}
	}
}

// retrieve a string value from the registry
LPTSTR RegQueryStringValue(HKEY hKey, LPCTSTR pszSubKey, LPCTSTR pszValueName, LPTSTR pszBuffer, int nBufferSize, LPTSTR pszDefaultValue)
{
	DWORD dwDataType;
	DWORD dwDataSize = (DWORD) nBufferSize;
	HKEY hSubKey = NULL;
	LONG lErrorCode;
	
	if(pszSubKey)
	{
		lErrorCode = RegOpenKeyEx(hKey, pszSubKey, 0, KEY_QUERY_VALUE, &hSubKey);
		
		if(lErrorCode != ERROR_SUCCESS)
		{
			//StringCchCopy(pszBuffer, nBufferSize, pszDefaultValue);
			return pszBuffer;
		}
		
		hKey = hSubKey;
	}
	
	lErrorCode = RegQueryValueEx(hKey, pszValueName, NULL, &dwDataType,
		(LPBYTE) pszBuffer, &dwDataSize);
	
	if(hSubKey)
		RegCloseKey(hSubKey);
		
	//if(dwDataType != REG_SZ || lErrorCode != ERROR_SUCCESS)
		//StringCchCopy(pszBuffer, nBufferSize, pszDefaultValue);
	
	return pszBuffer;
}

string IntToString(int input)
{
	char buffer[32];
	itoa( input, buffer, 10 );
	return (string) buffer;
}

